#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
stati.exceptions
~~~~~~~~~~~~~~~~~~~

Stati exceptions module

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/stati-python
"""

class InvalidData(Exception):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
stati.client
~~~~~~~~~~~~

Client for GottWall

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/stati-python
"""
import logging
import json
import datetime
import time
import redis
from random import randint


logger = logging.getLogger('stati')


class Client(object):
    """Base client
    """
    def __init__(self, project, private_key, public_key):
        self._project = project
        self._private_key = private_key
        self._public_key = public_key

    def serialize(self, name, timestamp, value, filters={}):
        """Make data bucket

        :param data: dict of data
        """
        return json.dumps(
            {"name": name,
             "timestamp": timestamp.strftime("%Y-%m-%dT%H:%M:%S"),
             "filters": filters,
             "value": value,
             "random": time.time()})

    def incr(self, name, timestamp=datetime.datetime.now(), value=1,
             filters={}):
        """Add data incrementation

        :param name:
        :param timestamp:
        :param value:
        :param filters:
        """
        raise NotImplementedError


class RedisClient(Client):
    """GottWall client that send data to redis pub/sub
    """

    def __init__(self, project, private_key, public_key,
                 host='localhost', port=6379, password=None,
                 db=0):
        super(RedisClient, self).__init__(project, private_key, public_key)
        self._host = host
        self._port = port
        self._password = password
        self._db = db
        self._pool = None
        self._key = None
        self._channel = None
        self._connection_pool = None
        self._redis = None

    @property
    def channel(self):
        """Make channel name
        """
        if not self._channel:
            self._channel = "gottwall:{0}:{1}:{2}".format(self._project, self._public_key, self._private_key)
        return self._channel

    @property
    def data_key(self):
        """Make key that store
        """
        if not self._key:
            self._key = "gottwall:{0}:{1}".format(self._project, self._public_key, self._private_key)
        return self._key

    @property
    def redis(self):
        if not self._connection_pool:
            self._connection_pool = redis.ConnectionPool(host=self._host, port=self._port,
                                                     db=self._db, password=self._password)
        if not self._redis:
            self._redis = redis.Redis(connection_pool=self._connection_pool)

        return self._redis

    def incr(self, name, timestamp=None, value=1, filters={}):
        timestamp = timestamp or datetime.datetime.now()
        try:
            redis = self.redis

            redis.sadd(self.data_key, self.serialize(name, timestamp, value, filters))
            redis.publish(self.channel, json.dumps({"type": "notification"}))
        except Exception, e:
            print(e)
            logger.warn(e)

    def serialize(self, name, timestamp, value, filters={}):
        """Make data bucket

        :param data: dict of data
        """
        return json.dumps(
            {"name": name,
             "timestamp": timestamp.strftime("%Y-%m-%dT%H:%M:%S"),
             "filters": filters,
             "value": value,
             "random": "{0}{1}".format(time.time(), str(randint(1, 1000)))})




#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
stati-redis-python
~~~~~~~~~~~~~~~~~~

Python client with redis transport for GottWall statistics aggregator

:copyright: (c) 2012 - 2013 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""


import sys
import os
from setuptools import  setup

try:
    readme_content = open(os.path.join(os.path.abspath(
        os.path.dirname(__file__)), "README.rst")).read()
except Exception, e:
    print(e)
    readme_content = __doc__

VERSION = "0.0.6"


def run_tests():
    from tests import suite
    return suite()

py_ver = sys.version_info

#: Python 2.x?
is_py2 = (py_ver[0] == 2)

#: Python 3.x?
is_py3 = (py_ver[0] == 3)

tests_require = [
    'nose',
    'unittest2']

install_requires = [
    "redis"]

setup(
    name="stati-redis",
    version=VERSION,
    description="Python client with redis transport for GottWall statistics aggregator",
    long_description=readme_content,
    author="Alex Lispython",
    author_email="alex@obout.ru",
    maintainer="Alexandr Lispython",
    maintainer_email="alex@obout.ru",
    url="https://github.com/GottWall/stati-redis-python",
    packages=["stati_redis"],
    install_requires=install_requires,
    tests_require=tests_require,
    license="BSD",
    platforms = ['Linux', 'Mac'],
    classifiers=[
        "Environment :: Web Environment",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries"
        ],
    test_suite = '__main__.run_tests')
